import java.awt.Component;
import java.awt.Frame;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.concurrent.ExecutionException;

import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.SwingWorker;

public class WaitDialog extends JDialog {

	public static WaitDialog waitDialog = null;
	
	private boolean disposed = false;
	private boolean displayed = false;
	private WorkerThread worker = null;
	
	WaitDialog(Frame parent, String text, SwingWorkerInterface in){
		super(parent, true);
		worker = new WorkerThread(in);
		setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		addWindowListener(new WindowAdapter() {
			@Override
			public void windowOpened(WindowEvent e) {	
				worker.execute();
			}

			@Override
			public void windowClosing(WindowEvent e) {
				disposeWaitDialog();
			}
		});

		this.setResizable(false);

		JLabel message = new JLabel();
		message.setText(text);
		
		JProgressBar pb = new JProgressBar();
		pb.setIndeterminate(true);
		
		// set size and location
		setSize(200, 100);
		setLocationRelativeTo(parent);

		JPanel panel = new JPanel();
		panel.add(message);
		panel.add(pb);
		
		add(panel);
	}

	public void showWaitDialog(){

		if(displayed == true){
			return;
		}

		if(disposed == true){
			disposed = false;
			return;
		}
		
		disposed = false;
		displayed = true;
		setVisible(true);
	}
	
	public void disposeWaitDialog(){
		
		if(disposed == true){
			return;
		}
		
		if(displayed == true){
			displayed = false;
			setVisible(false);
			return;
		}
		
		disposed = true;
		displayed = false;
	}
	
	public static Object showWaitDialog(Component parent, String text, SwingWorkerInterface in) throws InterruptedException, ExecutionException {
		if (parent == null) {
			waitDialog = new WaitDialog(JOptionPane.getRootFrame(), text, in);
		} else {
			waitDialog = new WaitDialog(JOptionPane.getFrameForComponent(parent), text, in);
		}
		
		

		while(!waitDialog.worker.isDone()){
			System.out.println("about to show");
			waitDialog.showWaitDialog();
			System.out.println("done showing");
		}
		
		waitDialog.dispose();
		return waitDialog.worker.get();
	}
	
	class WorkerThread extends SwingWorker<Throwable, Void> {

		private SwingWorkerInterface in = null;
		
		WorkerThread(SwingWorkerInterface in){
			this.in = in;
		}
		
		public Throwable doInBackground(){
				try {
					System.out.println("about to do work");
					in.workToDo();
					System.out.println("done work no exception");
				} catch (Throwable e) {
					System.out.println("done work with exception");
					return e;
				}
				return null;
			}
		
		public void done(){
				System.out.println("about to dispose");
				waitDialog.disposeWaitDialog();
				System.out.println("disposed");
		}
	}
}
